//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include "MiraGrid.h"
#pragma package(smart_init)

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int FontSize;
__fastcall TMiraGrid::TMiraGrid(TComponent* Owner)
  : TStringGrid(Owner)
{
  SetupCellArray();
  LastCombo = NULL;
  DoubleBuffered = true;
  CRow = -1;
  CCol = -1;
  FontSize = Font->Size;
}
//---------------------------------------------------------------------------
void __fastcall TMiraGrid::DrawCell(int ACol, int ARow, const TRect &ARect, Grids::TGridDrawState AState)
{
  if(Cell(ACol,ARow)->Canvas != NULL)
  {
    Canvas->CopyRect(ARect,Cell(ACol,ARow)->Canvas,Cell(ACol,ARow)->Canvas->ClipRect);
  }
  else
  {
    if(ARow != 0)
    {
      Canvas->Brush->Color = Cell(ACol,ARow)->BGColor;
    }
    Canvas->TextRect(ARect,ARect.Left+5,ARect.Top+5,Cells[ACol][ARow]);
  }
  Font->Size = FontSize;
}
void __fastcall TMiraGrid::ComboChanged(TObject* Sender)
{
  if(LastCombo != NULL)
  {
    Cells[CCol][CRow] = LastCombo->Text;
    LastCombo->Hide();
  }
  if(OnComboChangedCallback != NULL)
  {
    OnComboChangedCallback(Sender);
  }
}

void __fastcall TMiraGrid::CalendarChanged(TObject* Sender)
{
  if(LastCalendar != NULL)
  {
    Cells[CCol][CRow] = LastCalendar->Date.FormatString("yyyy-mm-dd");
    LastCalendar->Hide();
  }
}
void __fastcall TMiraGrid::EditChanged(TObject* Sender)
{
  if(LastEdit != NULL)
  {
    Cells[CCol][CRow] = LastEdit->Text;
    LastEdit->Hide();
    SetFocus();
  }
}
void __fastcall TMiraGrid::RowButtonClicked(TObject* Sender)
{
}


bool __fastcall TMiraGrid::SelectCell(int ACol, int ARow)
{
  CCol = ACol;
  CRow = ARow;
  if(OnSelectCellCallback != NULL)
  {
    OnSelectCellCallback(ACol,ARow);
  }
  if(LastCombo != NULL)
  {
    LastCombo->Hide();
  }
  if(LastCalendar != NULL)
  {
    LastCalendar->Hide();
  }
  if(LastEdit != NULL)
  {
    LastEdit->Hide();
  }

  if(ColCombo(ACol) != NULL)
  {
    LastCombo = ColCombo(ACol);
    LastCombo->Text = Cells[ACol][ARow];
    LastCombo->BoundsRect = CellRect(ACol,ARow);
    LastCombo->Height = CellRect(ACol,ARow).Height()*(LastCombo->Items->Count+1);
    if(LastCombo->Enabled)
    {
      LastCombo->Show();
      LastCombo->SetFocus();
    }
  }
  if(ColCalendar(ACol) != NULL)
  {
    LastCalendar = ColCalendar(ACol);
    LastCalendar->BoundsRect = CellRect(ACol,ARow);
    if(LastCalendar->Top+220 > Height)
    {
      LastCalendar->Top = Height - 220;
    }
    LastCalendar->Height = 200;
    LastCalendar->Width = 220;
    if(LastCalendar->Enabled)
    {
      LastCalendar->Show();
      LastCalendar->SetFocus();
    }
  }
  if(ColEdit(ACol) != NULL)
  {
    LastEdit = ColEdit(ACol);
    LastEdit->Text = Cells[ACol][ARow];
    LastEdit->BoundsRect = CellRect(ACol,ARow);
    if(LastEdit->Enabled)
    {
      LastEdit->Show();
      LastEdit->SetFocus();
    }
  }
  for(int i=0;i<ColCount;i++)
  {
    if(ColRowButton(i) != NULL)
    {
      TRect Rect = ColRowButton(i)->BoundsRect;
      Rect.Top = CellRect(ACol,ARow).Top;
      Rect.Bottom = CellRect(ACol,ARow).Bottom;
      ColRowButton(i)->BoundsRect = Rect;
    }
  }

  return true;
}

void __fastcall TMiraGrid::SetColCount(int Cols)
{
  while(Cols > ColCount)
  {
   CellArray.Length++;
   ColArray.Length++;
   ColArray.operator [](ColCount) = new TColumn();
   for(int row=0;row<RowCount;row++)
    {
      CellArray.operator [](ColCount).Length++;
      CellArray.operator [](ColCount).operator [](row) = new TCell();
    }
    ColCount++;
  }
  if(Cols < ColCount)
  {
    ColArray.Length = Cols;
    CellArray.Length = Cols;
    ColCount = Cols;
  }
}
void __fastcall TMiraGrid::SetRowCount(int Rows)
{
  while(Rows > RowCount)
  {
    for(int Col=0;Col<ColCount;Col++)
    {
      CellArray.operator [](Col).Length++;
      CellArray.operator [](Col).operator [](RowCount) = new TCell();
    }
    RowCount++;
  }
  while(Rows < RowCount)
  {
    for(int col=0;col<ColCount;col++)
    {
      CellArray.operator [](col).Length--;
    }
    RowCount = Rows;
  }
}
void __fastcall TMiraGrid::InsertRow(int AfterRow)
{
  SetRowCount(RowCount+1);
  for(int Row=RowCount-1;Row>AfterRow;Row--)
  {
    for(int Col=0;Col<ColCount;Col++)
    {
      Cells[Col][Row] = Cells[Col][Row-1];
      CellArray.operator [](Col).operator [](Row) = CellArray.operator [](Col).operator [](Row-1);
    }
  }
  for(int Col=0;Col<ColCount;Col++)
  {
    Cells[Col][AfterRow] = "";
    CellArray.operator [](Col).operator [](AfterRow) = new TCell();
  }
}
void __fastcall TMiraGrid::DeleteRow(int Row)
{
  if(RowCount == 2)
  {
    for(int Col=0;Col<ColCount;Col++)
    {
      Cells[Col][1] = "";
    }
  }
  else
  {
    for(int row=Row;row<RowCount-1;row++)
    {
      for(int Col=0;Col<ColCount;Col++)
      {
        Cells[Col][row] = Cells[Col][row+1];
        CellArray.operator [](Col).operator [](row) = CellArray.operator [](Col).operator [](row+1);
      }
    }
    SetRowCount(RowCount-1);
  }
  Invalidate();
}
void __fastcall TMiraGrid::SetupCellArray(void)
{
  CellArray.Length = RowCount;
  ColArray.Length = ColCount;
  for(int row=0;row<RowCount;row++)
  {
    CellArray.operator [](row).Length = ColCount;
    ColArray.operator [](row) = new TColumn();
  }
  for(int row=0;row<RowCount;row++)
  {
    for(int col=0;col<ColCount;col++)
    {
      CellArray.operator [](row).operator [](col) = new TCell();
    }
  }
}

TComboBox* __fastcall TMiraGrid::ColCombo(int Col)
{
  return ColArray.operator [](Col)->Combo;
}
TMonthCalendar* __fastcall TMiraGrid::ColCalendar(int Col)
{
  return ColArray.operator [](Col)->Calendar;
}
TEdit* __fastcall TMiraGrid::ColEdit(int Col)
{
  return ColArray.operator [](Col)->Edit;
}
TBitBtn* __fastcall TMiraGrid::ColRowButton(int Col)
{
  return ColArray.operator [](Col)->RowButton;
}
void __fastcall TMiraGrid::InsertCombo(int Col)
{
  ColArray.operator [](Col)->InsertCombo(this);
  ColCombo(Col)->OnExit = ComboChanged;
  ColCombo(Col)->Font->Size = Font->Size + 2;
}
void __fastcall TMiraGrid::InsertCalendar(int Col)
{
  ColArray.operator [](Col)->InsertCalendar(this);
  ColCalendar(Col)->OnDblClick = CalendarChanged;
  ColCalendar(Col)->Font->Size = Font->Size + 2;
}

void __fastcall TMiraGrid::InsertEdit(int Col)
{
  ColArray.operator [](Col)->InsertEdit(this);
  ColEdit(Col)->OnExit = EditChanged;
  ColEdit(Col)->Font->Size = Font->Size + 2;
}

void __fastcall TMiraGrid::InsertRowButton(int Col)
{
  ColArray.operator [](Col)->InsertRowButton(this);
  ColRowButton(Col)->BoundsRect = CellRect(Col,1);
  ColRowButton(Col)->Show();
}

TCell* __fastcall TMiraGrid::Cell(int Col,int Row)
{
  return CellArray.operator [](Col).operator [](Row);
}

void __fastcall TMiraGrid::SetCanvas(int Col, int Row, TCanvas *Canvas)
{
  CellArray.operator [](Col).operator [](Row)->Canvas = Canvas;
}
TCanvas* __fastcall TMiraGrid::GetCanvas(int Col, int Row)
{
  return CellArray.operator [](Col).operator [](Row)->Canvas;
}

int __fastcall TMiraGrid::FindStringInCol(String string, int Col)
{
  for(int i=0;i<RowCount;i++)
  {
    if(Cells[Col][i] == string)
    {
      return i;
    }
  }
  return -1;
}


namespace Miragrid
{
  void __fastcall PACKAGE Register()
  {
     TComponentClass classes[1] = {__classid(TMiraGrid)};
     RegisterComponents("Mira", classes, 0);
  }
}
//---------------------------------------------------------------------------




