unit UnitMain;

interface

{ 20040120  ritsaert@continuit.nl     Initial version.
  20040122  ritsaert@continuit.nl     Cleanup up example. }

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    btnTestVectors: TButton;
    mmResult: TMemo;
    btnTestSpeed: TButton;
    btnTestNoWrite: TButton;
    btnTestCopy: TButton;
    procedure btnTestVectorsClick(Sender: TObject);
    procedure btnTestSpeedClick(Sender: TObject);
    procedure btnTestNoWriteClick(Sender: TObject);
    procedure btnTestCopyClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  UnitHelix;

function CPUTimestamp: Int64;
asm
  db $0F, $31; // RDTSC
end;

{$R *.dfm}


procedure TForm1.btnTestVectorsClick(Sender: TObject);
var
  H: THelix;
  N: THelixBlock;
  S: string;
  B: array of Byte;
  Lm4: Longword;
begin
  with mmResult.Lines do begin
    Clear;
    Add('Test Vector 1.');
    Add('Key    ='); // zero length key
    HelixSetKey(H,H,0);

    // Check the key
    SetLength(S,2*sizeof(H.K)); BinToHex(PChar(@H.K),PChar(S),sizeof(H.K)); Add('ExpKey = ' + S);
    Assert(LowerCase(S) = 'a93b6e32bc234f6c326c0f8274ffa241e3da577def7c1b64af787c38dcefe3de');

    // CLear Nonce
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Encrypt 10 zero bytes
    SetLength(B,10); HexToBin('00000000000000000000',PChar(@B[0]),Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Lm4 := HelixEncrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Cipher = ' + S);
    Assert(Lowercase(S) = '7044c9be48ae892266e4');

    // And compute the MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');

    // Decrypt encrypted bytes
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    Lm4 := HelixDecrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Assert(Lowercase(S) = '00000000000000000000');

    // And compute the MAC (again)
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');

    // Now set the key to the second test vector.
    Add('');
    Add('Test Vector 2.');
    SetLength(B,32);
    HexToBin('0000000001000000020000000300000004000000050000000600000007000000',PChar(@B[0]),Length(B));
    HelixSetKey(H,B[0],Length(B));
    // Check the key
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Key    = ' + S);
    SetLength(S,2*sizeof(H.K)); BinToHex(PChar(@H.K),PChar(S),sizeof(H.K)); Add('ExpKey = ' + S);
    Assert(LowerCase(S) = '6ee9a76cbd0bf620a6d9b75949d3399504f84ad68312f906edd1a6989ec89d45');

    // Nonce;
    HexToBin('00000000010000000200000003000000',PChar(@N[0]),sizeof(N));
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Data
    SetLength(B,32);
    HexToBin('0000000001000000020000000300000004000000050000000600000007000000',PChar(@B[0]),Length(B));
    HelixSetKey(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);

    // Crypt
    HelixStart(H,N);
    Lm4 := HelixEncrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Cipher = ' + S);
    Assert(Lowercase(S) = '7a72a75b6250380b69751cd128308d9a0c7446a3bf3f99e66556b9c118ca7d87');

    // MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = 'e4e54901c50b34e780c09c39b109a117');

    HexToBin('00000000010000000200000003000000',PChar(@N[0]),sizeof(N));
    HelixStart(H,N);
    Lm4 := HelixDecrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Assert(Lowercase(S) = '0000000001000000020000000300000004000000050000000600000007000000');

    // And compute the MAC (again)
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = 'e4e54901c50b34e780c09c39b109a117');

    // Now set the key to the third test vector.
    Add('');
    Add('Test Vector 3.');
    SetLength(B,5);
    HexToBin('48656c6978',PChar(@B[0]),Length(B));
    HelixSetKey(H,B[0],Length(B));
    // Check the key
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Key    = ' + S);
    SetLength(S,2*sizeof(H.K)); BinToHex(PChar(@H.K),PChar(S),sizeof(H.K)); Add('ExpKey = ' + S);
    Assert(LowerCase(S) = '6c1ed77acba3a1d28f1cd6206df115daf403284a739bb69f357a85f551321139');

    // Nonce;
    HexToBin('30313233343536373839616263646566',PChar(@N[0]),sizeof(N));
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Data
    SetLength(B,13);
    HexToBin('48656c6c6f2c20776f726c6421',PChar(@B[0]),Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);

    // Crypt
    HelixStart(H,N);
    Lm4 := HelixEncrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Cipher = ' + S);
    Assert(Lowercase(S) = '6c4c27b97a82a0c5802c23f20d');

    // MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '6c82d1aa3b905f12f1443fa7f6a101d2');

    HexToBin('30313233343536373839616263646566',PChar(@N[0]),sizeof(N));
    HelixStart(H,N);
    Lm4 := HelixDecrypt(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Assert(Lowercase(S) = '48656c6c6f2c20776f726c6421');

    // And compute the MAC (again)
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '6c82d1aa3b905f12f1443fa7f6a101d2');

  end;
end;

procedure TForm1.btnTestSpeedClick(Sender: TObject);
var
  H: THelix;
  N, MAC: THelixBlock;
  B, B2: array of Byte;
  i,j: Integer;
  T: Int64;
  F: Double;
const
  BlockSize = 256*1024;
  MinBytesPerMeas = 5000000;
  Key = 'Hello World';
begin
  SetThreadPriority(GetCurrentThread,THREAD_PRIORITY_TIME_CRITICAL);
  with mmResult.Lines do begin
    Clear;

    // And do some timing....
    HelixSetKey(H,Key[1],Length(Key));
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);

    SetLength(B,BlockSize);
    SetLength(B2,BlockSize);

    // -----------------------------------------
    Add('Without MAC');
    Add('Blocksize   Encryption  Decryption (cycles/byte)');
    j := 16;
    while j <= BlockSize do begin
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixEncrypt(H,B[0],j);
        Inc(i,j);
      until i >= MinBytesPerMeas;
      F := (CPUTimestamp - T) / i;
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixDecrypt(H,B[0],j);
        Inc(i,j);
      until i >= MinBytesPerMeas;
      T := 1000 * (CPUTimestamp - T) div i;
      Add(Format('%-10d%12.3f%12.3f',[j,F, T * 0.001]));
      j := j * 4;
    end;

    // -----------------------------------------
    Add('');
    Add('With MAC');
    Add('Blocksize   Encryption  Decryption (cycles/byte)');
    j := 16;
    while j <= BlockSize do begin
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixEncrypt(H,B[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      F := (CPUTimestamp - T) / i;
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixDecrypt(H,B[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      T := 1000 * (CPUTimestamp - T) div i;
      Add(Format('%-10d%12.3f%12.3f',[j,F, T * 0.001]));
      j := j * 4;
    end;

    // -----------------------------------------
    Add('');
    Add('With MAC, Copy');
    Add('Blocksize   Encryption  Decryption (cycles/byte)');
    j := 16;
    while j <= BlockSize do begin
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixEncryptCopy(H,B[0],B2[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      F := (CPUTimestamp - T) / i;
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixDecryptCopy(H,B2[0],B[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      T := 1000 * (CPUTimestamp - T) div i;
      Add(Format('%-10d%12.3f%12.3f',[j,F, T * 0.001]));
      j := j * 4;
    end;

    // -----------------------------------------
    Add('');
    Add('Without MAC, Copy');
    Add('Blocksize   Encryption  Decryption (cycles/byte)');
    j := 16;
    while j <= BlockSize do begin
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixEncryptCopy(H,B[0],B2[0],j);
        Inc(i,j);
      until i >= MinBytesPerMeas;
      F := (CPUTimestamp - T) / i;
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixDecryptCopy(H,B2[0],B[0],j);
        Inc(i,j);
      until i >= MinBytesPerMeas;
      T := 1000 * (CPUTimestamp - T) div i;
      Add(Format('%-10d%12.3f%12.3f',[j,F, T * 0.001]));
      j := j * 4;
    end;

    // -----------------------------------------
    Add('');
    Add('MAC only');
    Add('Blocksize   Encryption  Decryption (cycles/byte)');
    j := 16;
    while j <= BlockSize do begin
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixEncryptNoWrite(H,B[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      F := (CPUTimestamp - T) / i;
      i := 0; T := CPUTimestamp;
      repeat
        HelixStart(H,N);
        HelixMac(H,MAC,HelixDecryptNoWrite(H,B[0],j));
        Inc(i,j);
      until i >= MinBytesPerMeas;
      T := 1000 * (CPUTimestamp - T) div i;
      Add(Format('%-10d%12.3f%12.3f',[j,F, T * 0.001]));
      j := j * 4;
    end;
  end;
  SetThreadPriority(GetCurrentThread,THREAD_PRIORITY_NORMAL);
end;

procedure TForm1.btnTestNoWriteClick(Sender: TObject);
var
  H: THelix;
  N: THelixBlock;
  S: string;
  B: array of Byte;
  Lm4: Longword;
begin
  with mmResult.Lines do begin
    Clear;
    Add('Test Vector 1.');
    Add('Key    ='); // zero length key
    HelixSetKey(H,H,0);

    // Check the key
    SetLength(S,2*sizeof(H.K)); BinToHex(PChar(@H.K),PChar(S),sizeof(H.K)); Add('ExpKey = ' + S);
    Assert(LowerCase(S) = 'a93b6e32bc234f6c326c0f8274ffa241e3da577def7c1b64af787c38dcefe3de');

    // CLear Nonce
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Encrypt 10 zero bytes
    SetLength(B,10); HexToBin('00000000000000000000',PChar(@B[0]),Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Lm4 := HelixEncryptNoWrite(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B));
    Assert(Lowercase(S) = '00000000000000000000');

    // And compute the MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');

    // CLear Nonce
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Set decrypted string (manually)
    SetLength(B,10); HexToBin('7044c9be48ae892266e4',PChar(@B[0]),Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Cipher = ' + S);
    Lm4 := HelixDecryptNoWrite(H,B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B));
    Assert(Lowercase(S) = '7044c9be48ae892266e4');

    // And compute the MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');
  end;
end;

procedure TForm1.btnTestCopyClick(Sender: TObject);
var
  H: THelix;
  N: THelixBlock;
  S: string;
  B, B2: array of Byte;
  Lm4: Longword;
begin
  with mmResult.Lines do begin
    Clear;
    Add('Test Vector 1.');
    Add('Key    ='); // zero length key
    HelixSetKey(H,H,0);

    // Check the key
    SetLength(S,2*sizeof(H.K)); BinToHex(PChar(@H.K),PChar(S),sizeof(H.K)); Add('ExpKey = ' + S);
    Assert(LowerCase(S) = 'a93b6e32bc234f6c326c0f8274ffa241e3da577def7c1b64af787c38dcefe3de');

    // Clear Nonce
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('Nonce  = ' + S);

    // Encrypt 10 zero bytes
    SetLength(B,10); HexToBin('00000000000000000000',PChar(@B[0]),Length(B));
    SetLength(B2,10); System.FillChar(B2[0],Length(B2),$55); // fill with garbage for testing.
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Lm4 := HelixEncryptCopy(H,B[0],B2[0],Length(B)); // !!!!
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B2));
    Assert(Lowercase(S) = '00000000000000000000');
    SetLength(S,2*Length(B2)); BinToHex(PChar(@B2[0]),PChar(S),Length(B2)); Add('Cipher = ' + S);
    Assert(Lowercase(S) = '7044c9be48ae892266e4');

    // And compute the MAC
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');

    // Decrypt encrypted bytes
    System.FillChar(N,sizeof(N),0); HelixStart(H,N);
    Lm4 := HelixDecryptCopy(H,B2[0],B[0],Length(B));
    SetLength(S,2*Length(B)); BinToHex(PChar(@B[0]),PChar(S),Length(B)); Add('Plain  = ' + S);
    Assert(Lowercase(S) = '00000000000000000000');
    SetLength(S,2*Length(B2)); BinToHex(PChar(@B2[0]),PChar(S),Length(B2));
    Assert(Lowercase(S) = '7044c9be48ae892266e4');

    // And compute the MAC (again)
    HelixMac(H,N,Lm4);
    SetLength(S,2*sizeof(N)); BinToHex(PChar(@N),PChar(S),sizeof(N)); Add('MAC    = ' + S);
    Assert(Lowercase(S) = '65be7a60fd3b8a5e316180805632d810');
  end;
end;

end.
